/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui;

import java.awt.Event;
import teikade.sys.CodePath;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.sys.UseProperties;
import teikade.gui.component.DTextField;
import teikade.gui.editor.StatusViewer;
import teikade.util.ThreadUtil;

public class CodePathField extends DTextField implements Runnable, StatusViewer, UseProperties {
    private Browser observer = null;
    private String codePath = null;
    private String statusMessage = null;
    private String errorMessage = null;
    private boolean keepStatus = false;
    private boolean focused = false;
    private Thread eraser = null;

    public CodePathField() {
	super();
	setEditable(false);
	updateProperties();
    }

    public void setObserver(Browser observer) {
	this.observer = observer;
	setEditable(true);
	if (!SystemManager.onJDK1_2) {
	    setForeground(java.awt.Color.black);
	    setBackground(new java.awt.Color(135, 135, 135));
	}
    }

    public boolean action(Event event, Object arg) {
	if (observer == null) return false;
	String text = getText();
	if (text.length() == 0) {
	    setText(codePath);
	    return true;
	}
	String codePathNames[] = CodePath.pathToNames(text);
	CodePath cp = CodePath.complete(codePathNames);
	if (cp != null) {	// completed
	    select(cp);
	    return true;
	}
	if (!select(new CodePath(codePathNames))) return true;
	showCodePath(CodePath.printString(codePathNames));
	showError("incomplete! ");
	return true;
    }

    private boolean select(CodePath cp) {
	String saveCodePath = codePath;
	codePath = cp.printString();
	if (!observer.select(cp)) {
	    showCodePath(saveCodePath);
	    return false;
	}
	return true;
    }

    public void setText(String text) {
	if (!text.equals(getText())) super.setText(text);
    }

    public void showCodePath(String message) {
	eraseMessages();
	codePath = message;
	showMessage();
    }

    public void viewStatus(String message, boolean keep) {
	eraseMessages();
	statusMessage = message;
	keepStatus = keep;
	showMessage();
	if (!keep) startEraser();
    }

    public void showError(String message) {
	eraseMessages();
	errorMessage = message;
	showMessage();
	startEraser();
    }

    public void showMessage() {
	if (errorMessage != null) {
	    setText(errorMessage);
	} else if (!focused && statusMessage != null) {
	    setText(statusMessage);
	} else {
	    setText(codePath);
	}
    }

    public void startEraser() {
	eraser = new Thread(this,this.getClass().getName());
	eraser.start();
    }

    public void run() {
	try {
	    Thread.sleep(errorMessage != null ? 1000 : 3000);
	} catch (InterruptedException e) {
	}
	eraseMessages();
	showMessage();
    }

    public void eraseMessages() {
	if (eraser != null) {
	    if (eraser != Thread.currentThread()) {
		//eraser.stop();
		ThreadUtil.Stop(eraser);
	    }
	    eraser = null;
	}
	errorMessage = null;
	if (!keepStatus) statusMessage = null;
    }

    public boolean gotFocus(Event evt, Object arg) {
	eraseMessages();
	focused = true;
	showMessage();
	return super.gotFocus(evt, arg);
    }

    public boolean lostFocus(Event evt, Object arg) {
	eraseMessages();
	focused = false;
	showMessage();
	return super.lostFocus(evt, arg);
    }
}
