/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.gui;

import java.util.*;
import java.io.*;
import java.awt.*;
import teikade.sys.*;
import teikade.util.CodeConverter;
import teikade.gui.component.*;
import teikade.gui.dialog.*;
import teikade.gui.menu.TranscriptMenuBar;
import teikade.gui.editor.EditData;
import teikade.gui.editor.EditDataObserver;
import teikade.util.ThreadUtil;

public class Transcript extends DFrame implements EditDataObserver, Runnable, Observer {
    public static String titleString = "System Transcript";
    private static Transcript transcript;

    private static Thread messageThread;
    private static Vector messages = new Vector();

    private static TranscriptMenuBar menuBar;
    private static TranscriptTextArea textArea;
    private static byte buf[] = new byte[256];

    public static Transcript getTranscript() {
	if (transcript == null) {
	    transcript = new Transcript(titleString);
	}
	return transcript;
    }

    public static TranscriptMenuBar getTranscriptMenuBar() {
	return (TranscriptMenuBar)getTranscript().getMenuBar();
    }

    public static Frame getDefaultFrame() {
	return getTranscript();
    }

    public static void raise() {
	Transcript transcritp = getTranscript();
	transcript.show();
	transcript.requestFocus();
    }

    private Transcript(String title) {
	super(title);

	setBackground(Color.lightGray);
	menuBar = new TranscriptMenuBar();
	menuBar.addFrame(this);
	setMenuBar(menuBar);

	textArea = new TranscriptTextArea(15, 80);
	add("Center", textArea.getComponent());
	pack();
	move(0, 0);

	textArea.setObserver(this);

	TeikadeProperties.observers.addObserver(this);
    }

    public void start() {
	show();
	getToolkit().sync();
	Thread.yield();
	try {
	    Thread.sleep(100);
	} catch (InterruptedException e) {
	}
	if (messageThread == null) {
	    messageThread = new Thread(this,this.getClass().getName());
	    messageThread.setPriority(Thread.MAX_PRIORITY);
	    messageThread.start();
	}
    }

    private void stop() {
	if (messageThread != null) {
	    //messageThread.stop();
	    ThreadUtil.Stop(messageThread);
	    messageThread = null;
	}
    }

    public boolean close() {

	if (!TeikadeDialog.confirm(getDefaultFrame(),
		  "You're going to quit teikade system. Really?",
		  "Quit", "Cancel")) {
	    return false;
	}

	Enumeration en = TeikadeFrame.allFrames();
	while (en.hasMoreElements()) {
	    TeikadeFrame frame = (TeikadeFrame)en.nextElement();
	    if (!frame.tryClose()) {
		TeikadeDialog.information(this, "Could not close some browsers.\nQuit aborted.");
		return false;
	    }
	}

	Enumeration changes = SourceFileManager.changedSources();
	if (changes.hasMoreElements()) {
	    StringBuffer files = new StringBuffer("");
	    do {
		SourceFileManager sfm = (SourceFileManager)changes.nextElement();
		files.append("  ");
		files.append(sfm.absolutePath());
		files.append('\n');
	    } while (changes.hasMoreElements());
	    int chosen =  TeikadeDialog.choose(getDefaultFrame(),
		"Following files are changed:\n" + files.toString() +
		"Sync them?", "Sync and quit", "No sync and quit", "Cancel");
	    if (chosen == 3) return false;
	    if (chosen == 1) SourceFileManager.syncAllChangedSources();
	}

	stop();
	super.close();
	SystemManager.saveProperties();
	System.exit(0);
	// not reached
	return true;
    }

    public void update(Observable observable, Object arg) {
	if (observable instanceof TeikadeProperties) {
	    updateProperties(this);
	}
	if (menuBar instanceof UseProperties) {
	    ((UseProperties)menuBar).updateProperties();
	}
	int num = menuBar.countMenus();
	for (int i = 0; i < num; i++) {
	    Menu menu = menuBar.getMenu(i);
	    if (menu instanceof UseProperties) {
		((UseProperties)menu).updateProperties();
	    }
	}
	invalidate();
	layout();
    }

    public void updateProperties(Container container) {
	Component comps[] = container.getComponents();
	for (int i = 0; i < comps.length; i++) {
	    Component comp = comps[i];
	    if (comp instanceof Container) {
		updateProperties((Container)comp);
	    } else if (comp instanceof UseProperties) {
		((UseProperties)comp).updateProperties();
	    }
	}
    }

    public boolean updateEditData(EditData editData, int command) {
	switch (command) {
	case CLOSE:
	case QUIT:
	    return close();
	}
	return false;
    }

    public boolean action(Event ev, Object arg) {
	return menuBar.action(ev, arg, this);
    }

    public void message(String str) {
	putMessage(new StringBufferInputStream(str));
    }

    public void message(InputStream stream) {
	putMessage(stream);
    }

    public void error(String str) {
	SystemManager.beep();
	textArea.appendText(str);
    }

    private void putMessage(InputStream stream) {
	synchronized (messages) {
	    messages.addElement(stream);
	    //messageThread.resume();
	    ThreadUtil.Resume(messageThread);
	}
	Thread.yield();
	try {
	    Thread.sleep(10);
	} catch (InterruptedException e) {
	}
    }

    private InputStream getMessage() {
	synchronized (messages) {
	    if (messages.size() == 0) {
		return null;
	    } else {
		InputStream message = (InputStream)messages.firstElement();
		messages.removeElementAt(0);
		return message;
	    }
	}
    }

    public String getText() {
	return textArea.getText();
    }

    public void setWaitCursor() {
	setCursor(Frame.WAIT_CURSOR);
    }

    public void setDefaultCursor() {
	setCursor(Frame.DEFAULT_CURSOR);
    }

    public void run() {
	Thread currentThread = Thread.currentThread();
	if (currentThread == messageThread) {
	    while (messageThread != null) {
		InputStream stream;
		while ((stream = getMessage()) != null) {
		    try {
			int len;
			while ((len = stream.read(buf)) != -1) {
			    textArea.appendText(CodeConverter.toString(buf, 0, len));
			    Thread.sleep(10);
			}
		    } catch (IOException e) {
		    } catch (InterruptedException e) {
		    } finally {
			try {
			    stream.close();
			} catch (IOException e) {
			}
		    }
		}
		//messageThread.suspend();
		ThreadUtil.Suspend(messageThread);
	    }
	}
    }

    public void memoryMonitor() {
	MemoryMonitor.showOn();
    }

    public void properties() {
	PropertiesDialog dialog = PropertiesDialog.on(this);
	dialog.show();
	dialog.dispose();
    }

    public static void addProjectToMenu(String projname) {
	getTranscriptMenuBar().addProject(projname);
    }

    public boolean removeProject(String projname) {
	ProjectManager proj = SystemManager.findProject(projname);
	Enumeration frames = TeikadeFrame.allFrames();
	while (frames.hasMoreElements()) {
	    TeikadeFrame frame = (TeikadeFrame)frames.nextElement();
	    if (frame.current.projectManager() == proj) {
		TeikadeDialog.information(this, "Project browser on " +
			    projname +
			    " is still opened.\nClose it and try again.");
		return false;
	    }
	}
	if (TeikadeDialog.confirm(this, "remove " + projname + " project, really?")) {
	    proj.remove();
	    return true;
	}
	return false;
    }

    public static void main(String argv[]) {
	getTranscript().start();
    }
}

class TranscriptTextArea {
    private int rows, cols;
    private DTextArea textArea;
    public static int messageNumber = 0;
    private static int limitSize = 0;
    static {
	if (SystemManager.onWin32) {
	    limitSize = 25 * 1024;
	}
    }

    public TranscriptTextArea(int rows, int cols) {
	this.rows = rows;
	this.cols = cols;
	textArea = new DTextArea(rows, cols);
	textArea.setEditable(true);
    }

    void setObserver(EditDataObserver observer) {
	textArea.getEditData().setObserver(observer);
    }

    public Component getComponent() {
	return textArea;
    }

    public void appendText(String message) {
	if (limitSize > 0) {
	    limitText(message);
	}
	textArea.appendText(message);
    }

    private void limitText(String message) {
	String oldText = textArea.getText();
	int oldLength = oldText.length();
	int addLength = message.length();
	if (oldLength + addLength >= limitSize) {
	    char c;
	    int cut = Math.min(oldLength, limitSize);
	    while (cut > 0) {
		cut--;
		c = oldText.charAt(cut);
		if (c == '\r' || c == '\n') {
		    cut++;
		    break;
		}
	    }
	    if (cut <= 0) cut = Math.min(oldLength, limitSize);
	    textArea.setText(oldText.substring(cut));
	    DTextArea ta = new DTextArea(rows, cols);
	    ta.setText(oldText.substring(0, cut));
	    Frame frame = new DFrame(Transcript.titleString
				    + " (old messages #"
				    + (++messageNumber)
				    + ")");
	    frame.add("Center", ta);
	    frame.pack();
	    frame.show();
	    try {
		Thread.sleep(100);
	    } catch (InterruptedException e) {
	    }
	    Transcript.raise();
	}
    }

    public String getText() {
	return textArea.getText();
    }
}
