/*
 * Copyright (c) 1997-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.awt.*;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.util.TextUtil;

// thanks for JDK bug patch on Win32 by Hideyo Namimoto
class Win32TextComponentPatch {
    private int adjusts[], initAdjust, endOfAdjust;
    private int doubleWidthes[], endOfDoubleWidthes;
    private static final boolean DO_PATCH_106KEYBOARD =
	"enable".equals(TeikadeProperties.getString("patch.Win32.Event.key.106keyboard"));
    private static final boolean DO_PATCH_FULLWIDTH =
	!"disable".equals(TeikadeProperties.getString("patch.Win32.TextComponent.char.doublewidth"));

    Win32TextComponentPatch() {
	this(32);
    }

    Win32TextComponentPatch(int lineFeedBufferInitialSize) {
	initAdjust = lineFeedBufferInitialSize;
	initialize();
    }

    synchronized void initialize() {
	adjusts = new int[initAdjust];
	doubleWidthes = new int[initAdjust];
	endOfAdjust = -1;
	endOfDoubleWidthes = -1;
    }

    synchronized void setText(String text) {
	initialize();
	if (text == null) return;
	int eot = text.length();
	TextUtil textUtil = TextUtil.getInstance();
	for (int i = 0; i < eot; i++) {
	    char c = text.charAt(i);
	    if (c == '\n') {
		addAdjust(i);
	    } else if (c >= 0x80 && DO_PATCH_FULLWIDTH) {
		if (textUtil.isDoubleWidth(c)) {
		    addDoubleWidth(i);
		}
	    }
	}
    }

    private void addDoubleWidth(int position) {
	if (++endOfDoubleWidthes >= doubleWidthes.length) {
	    int newAdjust[] = new int[endOfDoubleWidthes * 2];
	    System.arraycopy(doubleWidthes, 0, newAdjust, 0, endOfDoubleWidthes);
	    doubleWidthes = newAdjust;
	}
	doubleWidthes[endOfDoubleWidthes] = position;
	addAdjust(position);
    }

    private void addAdjust(int position) {
	if (++endOfAdjust >= adjusts.length) {
	    int newAdjust[] = new int[endOfAdjust * 2];
	    System.arraycopy(adjusts, 0, newAdjust, 0, endOfAdjust);
	    adjusts = newAdjust;
	}
	adjusts[endOfAdjust] = position;
    }

    synchronized int adjustPoint(int position) {
	int min = 0, max = endOfAdjust + 1, adjust = max / 2;
	while (min < max) {
	    if (position <= adjusts[adjust]) {
		max = adjust;
	    } else {
		min = adjust + 1;
	    }
	    adjust = (min + max) / 2;
	}
	return adjust;
    }

    synchronized int adjustRange(int position) {
	int min = 0, max = endOfDoubleWidthes + 1, adjust = max / 2;
	while (min < max) {
	    if (position <= doubleWidthes[adjust]) {
		max = adjust;
	    } else {
		min = adjust + 1;
	    }
	    adjust = (min + max) / 2;
	}
	return adjust;
    }

    synchronized int adjustForGetSelection(int position) {
	int min = 0, max = endOfAdjust + 1, adjust = max / 2;
	while (min < max) {
	    if (position <= adjusts[adjust] + adjust) {
		max = adjust;
	    } else {
		min = adjust + 1;
	    }
	    adjust = (min + max) / 2;
	}
	return adjust;
    }

    public final void patchEvent(Event e) {
	if (DO_PATCH_106KEYBOARD) {
	    int lkeyCode = e.key;
	    int lmodifiers = e.modifiers;
	    switch (lmodifiers){
	    case Event.SHIFT_MASK:
		switch (lkeyCode){
		case 38: // SHIFT+7
		  lkeyCode = (int)'\'';lmodifiers = 0;break;
		case 40: // SHIFT+9
		  lkeyCode = (int)')';lmodifiers = 0;break;
		case 41: // SHIFT+0
		  lkeyCode = (int)'_';lmodifiers = 0;break;
		case 42: // SHIFT+8
		  lkeyCode = (int)'(';lmodifiers = 0;break;
		case 58: // SHIFT+; and SHIFT+: (double assign)
		  lkeyCode = (int)'+';lmodifiers = 0;break;
		case 64: // SHIFT+2 and SHIFT+@ (double assign)
		  lkeyCode = (int)'"';lmodifiers = 0;break;
		case 94: // SHIFT+6 and SHIFT+^ (double assign)
		  lkeyCode = (int)'&';lmodifiers = 0;break;
		case 95: // SHIFT+-
		  lkeyCode = (int)'=';lmodifiers = 0;break;
		}
		break;
	    case Event.CTRL_MASK | Event.SHIFT_MASK:
		switch (lkeyCode){
		case 58: // CTRL+SHIFT+:
		  lkeyCode = (int)'*';lmodifiers = 0;break;
		case 64: // CTRL+SHIFT+@
		  lkeyCode = (int)'`';lmodifiers = 0;break;
		case 94: // CTRL+SHIFT+^
		  lkeyCode = (int)'~';lmodifiers = 0;break;
		case 124: // CTRL+SHIFT+\
		  lkeyCode = (int)'_';lmodifiers = 0;break;
		}
		break;
	    }
	    e.key = lkeyCode;
	    e.modifiers = lmodifiers;
	}
    }
}
