/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.menu;

import java.awt.*;
import java.util.Enumeration;
import teikade.sys.*;
import teikade.gui.*;
import teikade.gui.component.*;
import teikade.gui.dialog.MethodOrder;
import teikade.lang.AccessFlags;
import teikade.lang.MethodPrototype;

public class BrowserMenuBar extends TeikadeMenuBar {

    private DMenu optionMenu;
    private DMenu addPrototypeMenu;
    private MethodOrderMenu methodOrderMenu;

    protected final static String MENU_GO_TO_LINE      = "go to line";
    protected final static String MENU_SPAWN_CHANGES   = "spawn changes";
    protected final static String MENU_SPAWN_FILE      = "spawn file";
    protected final static String MENU_NEW_CLASS       = "new class";
    protected final static String MENU_NEW_INTERFACE   = "new interface";
    protected final static String MENU_NEW_CONSTRUCTOR = "new constructor";
    protected final static String MENU_NEW_METHOD      = "new method";
    protected final static String MENU_SET_PROTOTYPE   = "set prototype";
    protected final static String MENU_NEW_PACKAGE     = "new package";
    protected final static String MENU_ORDER           = "order";
    protected final static String MENU_CHANGE_ORDER    = "change order";
    protected final static String MENU_REMOVE_PACKAGE  = "remove package";
    protected final static String MENU_REMOVE_CLASS    = "remove class";
    protected final static String MENU_REMOVE_METHOD   = "remove method";
    protected final static String MENU_RENAME_PACKAGE  = "rename package";
    protected final static String MENU_RENAME_CLASS    = "rename class";
    protected final static String MENU_RENAME_METHOD   = "rename method";

    protected final static String MENU_HIDE_CLASS_METHODS     = "hide class methods";
    protected final static String MENU_HIDE_INSTANCE_METHODS  = "hide instance methods";
    protected final static String MENU_HIDE_PUBLIC_METHODS    = "hide public methods";
    protected final static String MENU_HIDE_FRIENDLY_METHODS  = "hide friendly methods";
    protected final static String MENU_HIDE_PROTECTED_METHODS = "hide protected methods";
    protected final static String MENU_HIDE_PRIVATE_METHODS   = "hide private methods";

    public BrowserMenuBar(DebugMenu debugMenu) {
	super(debugMenu);
    }

    protected DMenu createOptionMenu() {
	DMenu menu = super.createOptionMenu();
	menu.addSeparator();
	menu.addCheckboxItem(MENU_HIDE_CLASS_METHODS);
	menu.addCheckboxItem(MENU_HIDE_INSTANCE_METHODS);
	menu.addSeparator();
	menu.addCheckboxItem(MENU_HIDE_PUBLIC_METHODS);
	menu.addCheckboxItem(MENU_HIDE_FRIENDLY_METHODS);
	menu.addCheckboxItem(MENU_HIDE_PROTECTED_METHODS);
	menu.addCheckboxItem(MENU_HIDE_PRIVATE_METHODS);
	optionMenu = menu;
	return menu;
    }

    protected void createFileEditMenu(DMenu menu) {
        menu.add(MENU_GO_TO_LINE);
        menu.addSeparator();
        menu.add(MENU_SPAWN_CHANGES);
        menu.add(MENU_SPAWN_FILE);
        menu.addSeparator();
    }

    protected void createPackageEditMenu(DMenu menu) {
	menu.add(MENU_NEW_PACKAGE);
	menu.add(MENU_REMOVE_PACKAGE);
	menu.add(MENU_RENAME_PACKAGE);
	menu.addSeparator();
    }

    protected void createClassEditMenu(DMenu menu) {
	menu.add(MENU_NEW_CLASS);
	menu.add(MENU_NEW_INTERFACE);
	menu.add(MENU_REMOVE_CLASS);
	menu.add(MENU_RENAME_CLASS);
	menu.addSeparator();
    }

    protected void createMethodEditMenu(DMenu menu) {
	addPrototypeMenu = new DMenu(MENU_SET_PROTOTYPE);
	Enumeration prototypes = new MethodPrototype();
	while (prototypes.hasMoreElements()) {
	    String prototype = (String)prototypes.nextElement();
	    if (prototype == null) {
		addPrototypeMenu.addSeparator();
	    } else {
		addPrototypeMenu.add(prototype);
	    }
	}

	menu.add(methodOrderMenu = new MethodOrderMenu(MENU_ORDER));
	menu.add(MENU_CHANGE_ORDER);
	menu.addSeparator();
	menu.add(MENU_NEW_CONSTRUCTOR);
	menu.add(MENU_NEW_METHOD);
	menu.add(addPrototypeMenu);
	menu.add(MENU_REMOVE_METHOD);
	menu.add(MENU_RENAME_METHOD);
	menu.addSeparator();
    }

    public void updateMethodOrder(int order) {
	methodOrderMenu.update(order);
    }

    protected void updateFileMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.classManager() != null || cp.methodManager() != null) {
	    menu.enable(MENU_SPAWN_FILE);
	} else {
	    menu.disable(MENU_SPAWN_FILE);
	}
	super.updateFileMenuActivity(menu, frame, cp);
    }

    protected void updatePackageMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.projectManager() == null) {
	    menu.disable(MENU_NEW_PACKAGE);
	} else {
	    menu.enable(MENU_NEW_PACKAGE);
	}
	if (cp.packageManager() == null) {
	    menu.disable(MENU_REMOVE_PACKAGE);
	    menu.disable(MENU_RENAME_PACKAGE);
	} else {
	    menu.enable(MENU_REMOVE_PACKAGE);
	    menu.enable(MENU_RENAME_PACKAGE);
	}
	super.updatePackageMenuActivity(menu, frame, cp);
    }

    protected void updateClassMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.packageManager() == null) {
	    menu.disable(MENU_NEW_CLASS);
	    menu.disable(MENU_NEW_INTERFACE);
	} else {
	    menu.enable(MENU_NEW_CLASS);
	    menu.enable(MENU_NEW_INTERFACE);
	}
	if (cp.classManager() == null) {
	    menu.disable(MENU_REMOVE_CLASS);
	    menu.disable(MENU_RENAME_CLASS);
	} else {
	    menu.enable(MENU_REMOVE_CLASS);
	    menu.enable(MENU_RENAME_CLASS);
	}
	super.updateClassMenuActivity(menu, frame, cp);
    }

    protected void updateMethodMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	ClassManager cm = cp.classManager();
	if (cm == null) {
	    menu.disable(MENU_CHANGE_ORDER);
	    menu.disable(MENU_NEW_CONSTRUCTOR);
	    menu.disable(MENU_NEW_METHOD);
	    addPrototypeMenu.disable();
	} else {
	    boolean isInterface = false;
	    try {
		if (cm.spec().isInterface()) isInterface = true;
	    }
	    catch (TeikadeException e) {
	    }
	    menu.enable(MENU_CHANGE_ORDER);
	    if (isInterface) {
		menu.disable(MENU_NEW_CONSTRUCTOR);
		addPrototypeMenu.disable();
	    } else {
		menu.enable(MENU_NEW_CONSTRUCTOR);
		addPrototypeMenu.enable();
	    }
	    menu.enable(MENU_NEW_METHOD);
	}
	if (cp.methodManager() == null ||
	    cp.methodManager() instanceof HeaderManager) {
	    menu.disable(MENU_REMOVE_METHOD);
	    menu.disable(MENU_RENAME_METHOD);
	} else {
	    menu.enable(MENU_REMOVE_METHOD);
	    menu.enable(MENU_RENAME_METHOD);
	}
	super.updateMethodMenuActivity(menu, frame, cp);
    }

    protected void actionFileMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_GO_TO_LINE)) {
	    (new LocationFrame(cp)).open();
        } else if (label.equals(MENU_SPAWN_CHANGES)) {
            ChangeListFrame.openOn(cp);
        } else if (label.equals(MENU_SPAWN_FILE)) {
	    SourceFileManager sfm = cp.sourceFileManager();
            (new TeikadeFrame(new CodePath(sfm), "Source File '" + sfm.name() + "'")).open();
	} else {
	    super.actionFileMenu(label, frame, cp);
	}
    }

    protected void actionOptionMenu(String label, TeikadeFrame frame, CodePath cp) {
	try {
	    ((Browser)frame).updateMethods();
	}
	catch (TeikadeException e) {
	}
	super.actionOptionMenu(label, frame, cp);
    }

    protected void actionPackageMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_NEW_PACKAGE)) {
	    ((Browser)frame).newPackage(cp);
	} else if (label.equals(MENU_REMOVE_PACKAGE)) {
	    ((Browser)frame).remove(cp);
	}
	else if (label.equals(MENU_RENAME_PACKAGE)) {
	    ((Browser)frame).rename(cp);
	} else {
	    super.actionPackageMenu(label, frame, cp);
	}
    }

    protected void actionClassMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_NEW_CLASS)) {
	    ((Browser)frame).newClass(cp, false);
	} else if (label.equals(MENU_NEW_INTERFACE)) {
	    ((Browser)frame).newClass(cp, true);
	} else if (label.equals(MENU_REMOVE_CLASS)) {
	    ((Browser)frame).remove(cp);
	}
	else if (label.equals(MENU_RENAME_CLASS)) {
	    ((Browser)frame).rename(cp);
	} else {
	    super.actionClassMenu(label, frame, cp);
	}
    }

    protected void actionMethodMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_CHANGE_ORDER)) {
	    (new MethodOrder(frame, cp)).show();
	} else if (label.equals(MENU_NEW_CONSTRUCTOR)) {
	    ((Browser)frame).newMethod(cp, true);
	} else if (label.equals(MENU_NEW_METHOD)) {
	    ((Browser)frame).newMethod(cp, false);
	} else if (label.equals(MENU_REMOVE_METHOD)) {
	    ((Browser)frame).remove(cp);
	}
	else if (label.equals(MENU_RENAME_METHOD)) {
	    ((Browser)frame).rename(cp);
	} else {
	    super.actionMethodMenu(label, frame, cp);
	}
    }

    public boolean action(Event e, Object arg, TeikadeFrame frame, CodePath cp)
					throws TeikadeNoClassException {
	if ((e.target instanceof MenuItem) &&
	    ((MenuItem)e.target).getParent() == addPrototypeMenu) {
	    ((Browser)frame).newMethod(cp, MethodPrototype.getDescription((String)arg));
	    return true;
	}
	return super.action(e, arg, frame, cp);
    }

    public AccessFlags hideMethods() {
	AccessFlags hideMethods = new AccessFlags();
	if (optionMenu.getState(MENU_HIDE_CLASS_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.STATIC);
	}
	if (optionMenu.getState(MENU_HIDE_INSTANCE_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.NON_STATIC);
	}
	if (optionMenu.getState(MENU_HIDE_PUBLIC_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PUBLIC);
	}
	if (optionMenu.getState(MENU_HIDE_FRIENDLY_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.FRIENDLY);
	}
	if (optionMenu.getState(MENU_HIDE_PROTECTED_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PROTECTED);
	}
	if (optionMenu.getState(MENU_HIDE_PRIVATE_METHODS)) {
	    hideMethods.enableAccessFlags(AccessFlags.PRIVATE);
	}
	return hideMethods;
    }
}
