/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.menu;

import java.awt.Event;
import java.util.Observer;
import java.util.Observable;
import teikade.debug.DebuggerManager;
import teikade.gui.*;
import teikade.gui.component.*;

public class DebugMenu extends DMenu implements Observer {
    private Browser parent;
    private DMenuItem newDebuggerMenuItem;
    private SelectDebuggerMenu selectDebuggerMenu;
    private DMenuItem setBreakPointMenuItem;
    private DMenuItem resetBreakPointMenuItem;
    private DMenuItem clearBreakPointsMenuItem;
    private boolean selectedSourceCode;

    public DebugMenu(String name, Browser parent) {
	super(name);
	this.parent = parent;

	newDebuggerMenuItem      = new DMenuItem("new debugger");
	selectDebuggerMenu       = new SelectDebuggerMenu("select debugger");
	setBreakPointMenuItem    = new DMenuItem("set break point");
	resetBreakPointMenuItem  = new DMenuItem("reset break point");
	clearBreakPointsMenuItem = new DMenuItem("clear break points");

	add(newDebuggerMenuItem);
	add(selectDebuggerMenu);
	addSeparator();
	add(setBreakPointMenuItem);
	add(resetBreakPointMenuItem);
	add(clearBreakPointsMenuItem);

        selectedSourceCode = false;

	updateDebugger();
    }

    public boolean postEvent(Event ev) {
	if (ev.target == newDebuggerMenuItem) {
	    newDebugger();
	} else if (ev.target == setBreakPointMenuItem) {
	    setBreakPoint();
	} else if (ev.target == resetBreakPointMenuItem) {
	    resetBreakPoint();
	} else if (ev.target == clearBreakPointsMenuItem) {
	    clearBreakPoints();
	} else {
	    return false;
	}
	return true;
    }

    public void newDebugger() {
	try {
	    DebuggerManager newDebugger = DebuggerManager.createManager();
	    newDebugger.start();
	    parent.updateDebugger();
	    selectDebugger(newDebugger);
	} catch (java.io.IOException e) {
	    //parent.information("fail to create debugger connection");
	    System.out.println("fail to create debugger connection");
	}
    }

    public void updateDebugger() {
	selectDebuggerMenu.updateDebugger();
    }

    public void selectDebugger(DebuggerManager debugger) {
	selectDebuggerMenu.selectDebugger(debugger);
	updateActivity();
    }

    public DebuggerManager currentDebugger() {
	return selectDebuggerMenu.currentDebugger();
    }

    public void setBreakPoint() {
	parent.setBreakPoint(selectDebuggerMenu.currentDebugger());
    }

    public void resetBreakPoint() {
	parent.resetBreakPoint(selectDebuggerMenu.currentDebugger());
    }

    public void clearBreakPoints() {
	selectDebuggerMenu.currentDebugger().clearBreakPoints();
    }

    public void updateActivity(teikade.sys.CodePath cp) {
	selectedSourceCode = (cp.classManager() != null);
	updateActivity();
    }

    public void updateActivity() {
	if (selectedSourceCode &&
	    selectDebuggerMenu.currentDebugger() != null) {
	    setBreakPointMenuItem.enable();
	    resetBreakPointMenuItem.enable();
	} else {
	    setBreakPointMenuItem.disable();
	    resetBreakPointMenuItem.disable();
	}
	if (selectDebuggerMenu.currentDebugger() != null) {
	    clearBreakPointsMenuItem.enable();
	} else {
	    clearBreakPointsMenuItem.disable();
	}
    }

    public void update(Observable o, Object arg) {
	((teikade.net.Packet)arg).execute(parent);
    }
}

class SelectDebuggerMenu extends DMenu {
    private DCheckboxMenuItem noSelectDebuggerMenuItem;
    private DebuggerManager currentDebugger;

    SelectDebuggerMenu(String name) {
	super(name);
	noSelectDebuggerMenuItem = new DCheckboxMenuItem("(none)", null);
	currentDebugger = null;
    }

    void updateDebugger() {
	for (int i = countItems(); --i >= 0; ) {
	    remove(i);
	}
	add(noSelectDebuggerMenuItem);
	java.util.Enumeration en = DebuggerManager.list();
	while (en.hasMoreElements()) {
	    DebuggerManager debugger = (DebuggerManager)en.nextElement();
	    add(new DCheckboxMenuItem(debugger.name(), debugger));
	}
	((DebugMenu)getParent()).selectDebugger(currentDebugger);
    }

    void selectDebugger(DebuggerManager debugger) {
	if (currentDebugger != null) {
	    currentDebugger.deleteObserver((Observer)getParent());
	    currentDebugger = null;
	}
	for (int i = countItems(); --i >= 0; ) {
	    DCheckboxMenuItem menuItem = (DCheckboxMenuItem)getItem(i);
	    if (menuItem.getTarget() == debugger) {
		if (debugger != null) {
		    debugger.addObserver((Observer)getParent());
		}
		currentDebugger = debugger;
		menuItem.setState(true);
	    } else {
		menuItem.setState(false);
	    }
	}
    }

    DebuggerManager currentDebugger() {
	return currentDebugger;
    }

    public boolean postEvent(Event ev) {
	DCheckboxMenuItem selected = (DCheckboxMenuItem)ev.target;
	((DebugMenu)getParent()).selectDebugger((DebuggerManager)selected.getTarget());
	return true;
    }
}
