/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 *
 * Wed, 03 Nov 1999 10:24:17 +0000
 *	The history menu is offered by Bill Petheram.
 */
package teikade.gui.menu;

import teikade.sys.*;
import teikade.util.*;
import teikade.lang.*;
import teikade.gui.*;
import teikade.gui.dialog.TeikadeFileDialog;
import teikade.gui.component.*;
import teikade.gui.editor.EditData;
import teikade.gui.editor.EditProperties;
import java.util.Enumeration;
import java.awt.*;

public class TeikadeMenuBar extends DMenuBar {

    protected DMenu fileMenu;
    protected DMenu editMenu;
    protected DMenu optionMenu;
    protected DMenu projectMenu;
    protected DMenu packageMenu;
    protected DMenu classMenu;
    protected DMenu methodMenu;
    protected DMenu pluginMenu;
    protected DMenu historyMenu;
    protected TeikadeMessagesMenu messagesMenu;

    protected static final String MENU_FILE    = "File";
    protected static final String MENU_EDIT    = "Edit";
    protected static final String MENU_OPTION  = "Option";
    protected static final String MENU_PROJECT = "Project";
    protected static final String MENU_PACKAGE = "Package";
    protected static final String MENU_CLASS   = "Class";
    protected static final String MENU_METHOD  = "Method";
    protected static final String MENU_HISTORY = "History";

    protected final static String MENU_BEGINNING_OF_BLOCK = "beginning of block";
    protected final static String MENU_COMPILE            = "compile";
    protected final static String MENU_COMPILE_UPDATED    = "compile updated sources";
    protected final static String MENU_COPY               = "copy";
    protected final static String MENU_CUT                = "cut";
    protected final static String MENU_END_OF_BLOCK       = "end of block";
    protected final static String MENU_EXECUTE_WITH_ARGS  = "execute with args";
    protected final static String MENU_EXECUTE_WITH_HTML  = "execute with HTML";
    protected final static String MENU_EXECUTE            = "execute";
    protected final static String MENU_GREP               = "grep";
    protected final static String MENU_HIERARCHY          = "hierarchy";
    protected final static String MENU_IMPLEMENTERS       = "implementers";
    protected final static String MENU_PASTE              = "paste";
    protected final static String MENU_MACRO              = "macros ...";
    protected final static String MENU_PROPERTIES         = "properties";
    protected final static String MENU_CLOSE              = "close";
    protected final static String MENU_RAISE_TRANSCRIPT	  = "raise transcript";
    protected final static String MENU_REBUILD_DATABASE   = "rebuild database";
    protected final static String MENU_SEARCH_REPLACE     = "search/replace";
    protected final static String MENU_SENDERS            = "senders";
    protected final static String MENU_SELECT_BLOCK       = "select block";
    protected final static String MENU_SHIFT_BLOCK_RIGHT  = "shift block right {>>}";
    protected final static String MENU_SHIFT_BLOCK_LEFT   = "shift block left  {<<}";
    protected final static String MENU_SHIFT_LINE_RIGHT   = "shift lines right  >>";
    protected final static String MENU_SHIFT_LINE_LEFT    = "shift lines left   <<";
    protected final static String MENU_SPAWN_PROJECT	  = "spawn ";
    protected final static String MENU_SPAWN              = "spawn";
    protected final static String MENU_SYNC_DATABASE      = "sync database";
    protected final static String MENU_SYNC               = "sync";
    protected final static String MENU_USERS              = "users";
    protected final static String MENU_MESSAGES           = "messages";

    protected final static String MENU_PLUGIN             = "plugins ...";

    private static final String HISTORY_LIST_LINES = "attribute.history.lines";
    private int historyListPosition = 0;

    EditData editData = null;
    Replacer replacer = null;
    MacroEditor macroEditor = null;

    private static String buffer = new String();

    public TeikadeMenuBar() {
	this(null);
    }

    public TeikadeMenuBar(DebugMenu debugMenu) {
	super();

	fileMenu     = createFileMenu();
	editMenu     = createEditMenu();
	optionMenu   = createOptionMenu();
	projectMenu  = createProjectMenu();
	packageMenu  = createPackageMenu();
	classMenu    = createClassMenu();
	methodMenu   = createMethodMenu();
	historyMenu  = createHistoryMenu();

	if (fileMenu     != null) add(fileMenu);
	if (editMenu     != null) add(editMenu);
	if (debugMenu    != null) add(debugMenu);
	if (optionMenu   != null) add(optionMenu);
	if (projectMenu  != null) add(projectMenu);
	if (packageMenu  != null) add(packageMenu);
	if (classMenu    != null) add(classMenu);
	if (methodMenu   != null) add(methodMenu);
	if (historyMenu  != null) add(historyMenu);
    }

    public void close() {
	if (replacer != null) {
	    replacer.hide();
	    replacer.dispose();
	    replacer = null;
	}
    }

    public void setEditData(EditData editData) {
	this.editData = editData;
    }

    /**
     * update Menu activity
     */
    public void updateActivity(TeikadeFrame frame, CodePath cp) {
	if (   fileMenu != null)    updateFileMenuActivity(   fileMenu, frame, cp);
	if (   editMenu != null)    updateEditMenuActivity(   editMenu, frame, cp);
	if ( optionMenu != null)  updateOptionMenuActivity( optionMenu, frame, cp);
	if (projectMenu != null) updateProjectMenuActivity(projectMenu, frame, cp);
	if (packageMenu != null) updatePackageMenuActivity(packageMenu, frame, cp);
	if (  classMenu != null)   updateClassMenuActivity(  classMenu, frame, cp);
	if ( methodMenu != null)  updateMethodMenuActivity( methodMenu, frame, cp);
	if (historyMenu != null) updateHistoryMenuActivity(historyMenu, frame, cp);
    }

    public boolean action(Event e, Object arg, TeikadeFrame frame, CodePath cp)
					throws TeikadeNoClassException {
	if (e.target instanceof MenuItem) {
	    int saveCursor = frame.getCursorType();
	    frame.setCursor(Frame.WAIT_CURSOR);
	    frame.getToolkit().sync();
	    boolean done = true;
	    Menu menu = (Menu)(((MenuItem)e.target).getParent());
	    String label = arg.toString();
		 if (menu ==     fileMenu)      actionFileMenu(label, frame, cp);
	    else if (menu ==     editMenu)      actionEditMenu(label, frame, cp);
	    else if (menu ==   optionMenu)    actionOptionMenu(label, frame, cp);
	    else if (menu ==  projectMenu)   actionProjectMenu(label, frame, cp.projectCodePath());
	    else if (menu ==  packageMenu)   actionPackageMenu(label, frame, cp.packageCodePath());
	    else if (menu ==    classMenu)     actionClassMenu(label, frame, cp.classCodePath());
	    else if (menu ==   methodMenu)    actionMethodMenu(label, frame, cp);
	    else if (menu == messagesMenu)  messagesMenu.spawn(label, frame, cp);
	    else if (menu ==   pluginMenu)    actionPluginMenu(label, frame, cp);
	    else if (menu ==  historyMenu)   actionHistoryMenu(label, frame, cp);
	    else done = false;
	    frame.setCursor(saveCursor);
	    return done;
	} else {
	    return false;
	}
    }

    protected final DMenu createFileMenu() {
	DMenu menu = new DMenu(MENU_FILE);
	createFileEditMenu(menu);
	menu.add(MENU_RAISE_TRANSCRIPT);
	menu.add(MENU_SYNC_DATABASE);
	menu.add(MENU_CLOSE);
	return menu;
    }

    protected final DMenu createEditMenu() {
	DMenu menu = new DMenu(MENU_EDIT, true);
	menu.add(MENU_COPY);
	menu.add(MENU_CUT);
	menu.add(MENU_PASTE);
	menu.addSeparator();
	menu.add(MENU_SEARCH_REPLACE);
	menu.add(MENU_MACRO);
	menu.addSeparator();
	menu.add(MENU_PROPERTIES);
	return menu;
    }

    protected DMenu createOptionMenu() {
	DMenu menu = new DMenu(MENU_OPTION);
	pluginMenu = new DMenu(MENU_PLUGIN);
	PluginManager.addPluginLabels(pluginMenu);
	menu.add(pluginMenu);
	return menu;
    }

    protected final DMenu createProjectMenu() {
	DMenu menu = new DMenu(MENU_PROJECT);
	menu.add(MENU_SYNC);
	menu.add(MENU_GREP);
	menu.addSeparator();
	menu.add(MENU_REBUILD_DATABASE);
	menu.addSeparator();
	menu.add(MENU_SPAWN);
	return menu;
    }

    protected final DMenu createPackageMenu() {
	DMenu menu = new DMenu(MENU_PACKAGE);
	menu.add(MENU_SYNC);
	menu.add(MENU_COMPILE);
	menu.add(MENU_COMPILE_UPDATED);
	menu.add(MENU_GREP);
	menu.addSeparator();
	createPackageEditMenu(menu);
	menu.add(MENU_REBUILD_DATABASE);
	menu.addSeparator();
	menu.add(MENU_SPAWN);
	return menu;
    }

    protected final DMenu createClassMenu() {
	DMenu menu = new DMenu(MENU_CLASS);
	menu.add(MENU_SYNC);
	menu.add(MENU_COMPILE);
	menu.add(MENU_EXECUTE);
	menu.add(MENU_EXECUTE_WITH_ARGS);
	menu.add(MENU_EXECUTE_WITH_HTML);
	menu.add(MENU_GREP);
	menu.addSeparator();
	createClassEditMenu(menu);
	menu.add(MENU_SPAWN);
	menu.add(MENU_HIERARCHY);
	menu.add(MENU_IMPLEMENTERS);
	menu.add(MENU_USERS);
	return menu;
    }

    protected final DMenu createMethodMenu() {
	DMenu menu = new DMenu(MENU_METHOD);
	messagesMenu = new TeikadeMessagesMenu(MENU_MESSAGES);
	messagesMenu.disable();

	createMethodEditMenu(menu);
	menu.add(MENU_SPAWN);
	menu.add(MENU_HIERARCHY);
	menu.add(MENU_IMPLEMENTERS);
	menu.add(MENU_SENDERS);
	menu.add(messagesMenu);
	return menu;
    }

    protected final DMenu createHistoryMenu() {
	DMenu menu = new DMenu(MENU_HISTORY);
	return menu;
    }

    protected void createFileEditMenu(DMenu menu) {
    }

    protected void createPackageEditMenu(DMenu menu) {
    }

    protected void createClassEditMenu(DMenu menu) {
    }

    protected void createMethodEditMenu(DMenu menu) {
    }

    protected void updateFileMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
   }

    protected void updateEditMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	menu.enable(MENU_COPY);
	if (editData != null && editData.isEditable()) {
	    menu.enable(MENU_CUT);
	    menu.enable(MENU_PASTE);
	} else {
	    menu.disable(MENU_CUT);
	    menu.disable(MENU_PASTE);
	}
	/*
	if (cp.sourceFileManager() != null
	&&  editData != null && editData.isEditable()) {
	    menu.enable(MENU_SHIFT_BLOCK_RIGHT);
	    menu.enable(MENU_SHIFT_BLOCK_LEFT);
	    menu.enable(MENU_SHIFT_LINE_RIGHT);
	    menu.enable(MENU_SHIFT_LINE_LEFT);
	} else {
	    menu.disable(MENU_SHIFT_BLOCK_RIGHT);
	    menu.disable(MENU_SHIFT_BLOCK_LEFT);
	    menu.disable(MENU_SHIFT_LINE_RIGHT);
	    menu.disable(MENU_SHIFT_LINE_LEFT);
	}
	*/
	menu.enable(MENU_PROPERTIES);
	/*
	if (cp.classManager() != null) {
	    menu.enable(MENU_SELECT_BLOCK);
	    menu.enable(MENU_BEGINNING_OF_BLOCK);
	    menu.enable(MENU_END_OF_BLOCK);
	} else {
	    menu.disable(MENU_SELECT_BLOCK);
	    menu.disable(MENU_BEGINNING_OF_BLOCK);
	    menu.disable(MENU_END_OF_BLOCK);
	}
	*/
	if (replacer != null && replacer.isVisible()) replacer.updateActibity();
    }

    protected void updateOptionMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	PluginManager.updateMenuActivity(pluginMenu, frame, cp);
    }

    protected void updateProjectMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.projectManager() == null) {
	    menu.disable(MENU_SYNC);
	    menu.disable(MENU_GREP);
	    menu.disable(MENU_REBUILD_DATABASE);
	    menu.disable(MENU_SPAWN);
	} else {
	    menu.enable(MENU_SYNC);
	    if (Grep.canExecute()) {
		menu.enable(MENU_GREP);
	    } else {
		menu.disable(MENU_GREP);
	    }
	    menu.enable(MENU_REBUILD_DATABASE);
	    menu.enable(MENU_SPAWN);
	}
    }

    protected void updatePackageMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	if (cp.packageManager() == null) {
            menu.disable(MENU_SYNC);
	    menu.disable(MENU_COMPILE);
	    menu.disable(MENU_COMPILE_UPDATED);
            menu.disable(MENU_GREP);
            menu.disable(MENU_REBUILD_DATABASE);
            menu.disable(MENU_SPAWN);
        } else {
            menu.enable(MENU_SYNC);
	    if (Compile.canExecute(cp)) {
		menu.enable(MENU_COMPILE);
		menu.enable(MENU_COMPILE_UPDATED);
	    } else {
		menu.disable(MENU_COMPILE);
		menu.disable(MENU_COMPILE_UPDATED);
	    }
	    if (Grep.canExecute()) {
		menu.enable(MENU_GREP);
	    } else {
		menu.disable(MENU_GREP);
	    }
            menu.enable(MENU_REBUILD_DATABASE);
            menu.enable(MENU_SPAWN);
	}
    }

    protected void updateClassMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	menu.disable(MENU_COMPILE);
	menu.disable(MENU_EXECUTE);
	menu.disable(MENU_EXECUTE_WITH_ARGS);
	menu.disable(MENU_EXECUTE_WITH_HTML);
	menu.disable(MENU_GREP);
	menu.disable(MENU_IMPLEMENTERS);
	menu.enable(MENU_USERS);
	ClassManager cm = cp.classManager();
        if (cm == null) {
            menu.disable(MENU_SYNC);
	    menu.disable(MENU_COMPILE);
            menu.disable(MENU_GREP);
            menu.disable(MENU_SPAWN);
            menu.disable(MENU_HIERARCHY);
            menu.disable(MENU_IMPLEMENTERS);
            menu.disable(MENU_USERS);
        } else {
            menu.enable(MENU_SYNC);
	    if (Compile.canExecute(cp)) {
		menu.enable(MENU_COMPILE);
	    }
	    if (ClassManager.canExecute()) {
		new MenuUpdater(cm, menu).start();
	    }
            menu.enable(MENU_SPAWN);
            menu.enable(MENU_HIERARCHY);
	    if (Grep.canExecute()) {
		menu.enable(MENU_GREP);
		menu.enable(MENU_USERS);
		if (cm.isInterface()) {
		    menu.enable(MENU_IMPLEMENTERS);
		}
	    }
        }
    }

    protected void updateMethodMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	menu.disable(MENU_IMPLEMENTERS);
	menu.disable(MENU_SENDERS);
	menu.disable(MENU_MESSAGES);
	menu.disable(MENU_SPAWN);
	menu.disable(MENU_HIERARCHY);
	MethodManager mm = cp.methodManager();
        if (mm != null && !(mm instanceof HeaderManager)) {
	    menu.enable(MENU_SPAWN);
	    menu.enable(MENU_HIERARCHY);
	    if (ClassManager.canExecute()) {
		menu.enable(MENU_IMPLEMENTERS);
		menu.enable(MENU_SENDERS);
		messagesMenu.clear();
		try {
		    Enumeration messages = mm.messages();
		    while (messages.hasMoreElements()) {
			IdentifierSpec spec = (IdentifierSpec)messages.nextElement();
			if (spec.isMethod() >= IdentifierSpec.MAYBE) {
			    messagesMenu.add(spec);
			}
		    }
		} catch (DescriptionException e) {
		}
		if (messagesMenu.countItems() != 0) {
		    menu.enable(MENU_MESSAGES);
		}
	    }
	}
    }

    protected void updateHistoryMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
    int count = menu.countItems();
    String new_entry = cp.printString();
    for (int i = 0; i < count; i++)
    {
        String history = menu.getItem(i).getLabel();
        if (history.equals(new_entry))
        {
            // new_entry already in list. So return;
            return;
        }
    }
	MenuItem new_item = new MenuItem(new_entry);
	int listLines = TeikadeProperties.getInteger(HISTORY_LIST_LINES);
    if (historyListPosition == listLines)
    {
        historyListPosition = 0;
    }
    if ((historyListPosition < count) && (menu.getItem(historyListPosition) != null))
    {
        menu.remove(historyListPosition);
    }
    menu.insert(new_item, historyListPosition++);
    }

    protected void actionFileMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_RAISE_TRANSCRIPT)) {
	    Transcript.raise();
	} else if (label.equals(MENU_SYNC_DATABASE)) {
	    SystemManager.sync();
	} else if (label.equals(MENU_CLOSE)) {
	    frame.close();
	}
    }

    protected void actionEditMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (editData == null) {
	    return;
	}
	if (label.equals(MENU_COPY)) {
	    editData.copy();
	} else if (label.equals(MENU_CUT)) {
	    if (editData.isEditable()) {
		editData.cut();
	    }
	} else if (label.equals(MENU_PASTE)) {
	    if (editData.isEditable()) {
		editData.paste();
	    }
	} else if (label.equals(MENU_SEARCH_REPLACE)) {
	    if (replacer == null) {
		replacer = new Replacer(frame, editData);
	    }
	    replacer.show();
	} else if (label.equals(MENU_MACRO)) {
	    if (macroEditor == null) {
		macroEditor = new MacroEditor(frame, editData);
	    }
	    macroEditor.show();
	/*
	} else if (label.equals(MENU_SHIFT_BLOCK_RIGHT)) {
	    Editor.shiftBlockRight(editData);
	} else if (label.equals(MENU_SHIFT_BLOCK_LEFT)) {
	    Editor.shiftBlockLeft(editData);
	} else if (label.equals(MENU_SHIFT_LINE_RIGHT)) {
	    Editor.shiftLineRight(editData);
	} else if (label.equals(MENU_SHIFT_LINE_LEFT)) {
	    Editor.shiftLineLeft(editData);
	*/
	} else if (label.equals(MENU_PROPERTIES)) {
	    teikade.gui.editor.EditProperties.showDialogOn(frame);
	/*
	} else if (label.equals(MENU_SELECT_BLOCK)) {
	    Editor.selectBlock(editData);
	} else if (label.equals(MENU_BEGINNING_OF_BLOCK)) {
	    Editor.beginningOfBlock(editData);
	} else if (label.equals(MENU_END_OF_BLOCK)) {
	    Editor.endOfBlock(editData);
	*/
	}
    }

    protected void actionOptionMenu(String label, TeikadeFrame frame, CodePath cp) {
    }

    private void actionPluginMenu(String label, TeikadeFrame frame, CodePath cp) {
	PluginManager.invoke(label, frame, cp);
    }

    private void actionHistoryMenu(String label, TeikadeFrame frame, CodePath cp) {
        Transcript.getTranscript().message("actionHistoryMenu " + label + " " + frame + " " + cp.printString() + "\n");
        cp = new CodePath(label);
        if (frame instanceof Browser)
        {
            ((Browser) frame).select(cp);
        }
    }

    protected void actionProjectMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_SYNC)) {
	    frame.sync(cp);
	} else if (label.equals(MENU_GREP)) {
	    frame.grep(cp);
	} else if (label.equals(MENU_REBUILD_DATABASE)) {
	    frame.rebuild(cp);
	} else if (label.startsWith(MENU_SPAWN)) {
	    frame.spawn(cp);
	}
    }

    protected void actionPackageMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_SYNC)) {
	    frame.sync(cp);
	} else if (label.equals(MENU_COMPILE)) {
	    frame.compile(cp, false);
	} else if (label.equals(MENU_COMPILE_UPDATED)) {
	    frame.compile(cp, true);
	} else if (label.equals(MENU_GREP)) {
	    frame.grep(cp);
	} else if (label.equals(MENU_REBUILD_DATABASE)) {
	    frame.rebuild(cp);
	} else if (label.equals(MENU_SPAWN)) {
	    frame.spawn(cp);
	}
    }

    protected void actionClassMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_SYNC)) {
	    frame.sync(cp);
	} else if (label.equals(MENU_COMPILE)) {
	    frame.compile(cp, false);
	} else if (label.equals(MENU_EXECUTE)) {
	    cp.classManager().execute(null, frame.currentDebugger());
	} else if (label.equals(MENU_EXECUTE_WITH_ARGS)) {
	    String arg = frame.request("arguments:");
	    if (arg != null) {
		cp.classManager().execute(arg, frame.currentDebugger());
	    }
	} else if (label.equals(MENU_EXECUTE_WITH_HTML)) {
	    String htmlKey = "html." + cp.printString();
	    int sharp = htmlKey.indexOf('#');
	    if (sharp >= 0) {
		htmlKey = htmlKey.substring(0, sharp);
	    }
	    TeikadeFile file;
	    String defaultHtml = (String)TeikadeProperties.getString(htmlKey);
	    if (defaultHtml != null) {
		file = TeikadeFile.create(defaultHtml);
	    } else {
		file = TeikadeFile.create(cp.sourceFileManager().absolutePath(),
				cp.className() + ".html");
		defaultHtml = file.getPath();
	    }
	    TeikadeFileDialog fd = new TeikadeFileDialog(frame, "HTML file for execute");
	    fd.setDirectory(file.getParent());
	    fd.setFile(file.getName());
	    fd.setPostFix(".html");
	    String html = fd.request();
	    if (html != null) {
		file = TeikadeFile.create(html);
		if (file.isFile()) {
		    if (!html.equals(defaultHtml)) {
			TeikadeProperties.putWithoutNotify(htmlKey, html);
		    }
		    cp.classManager().executeWithHTML(html, frame.currentDebugger());
		} else {
		    frame.information(html + " is not a HTML file.");
		}
	    }
	} else if (label.equals(MENU_GREP)) {
	    frame.grep(cp);
	} else if (label.equals(MENU_SPAWN)) {
	    frame.spawn(cp);
	} else if (label.equals(MENU_HIERARCHY)) {
	    frame.spawnHierarchy(cp);
	} else if (label.equals(MENU_IMPLEMENTERS)) {
	    frame.spawnImplementers(cp);
	} else if (label.equals(MENU_USERS)) {
	    frame.spawnUsers(cp);
	}
    }

    protected void actionMethodMenu(String label, TeikadeFrame frame, CodePath cp) {
	if (label.equals(MENU_SPAWN)) {
	    frame.spawn(cp);
	} else if (label.equals(MENU_HIERARCHY)) {
	    frame.spawnHierarchy(cp);
	} else if (label.equals(MENU_IMPLEMENTERS)) {
	    frame.spawnImplementers(cp);
	} else if (label.equals(MENU_SENDERS)) {
	    frame.spawnSenders(cp);
	} else if (label.equals(MENU_MESSAGES)) {
	    // does not work
	    // messagesMenu.spawn(cp, frame);
	}
    }
}

class MenuUpdater implements Runnable {
    ClassManager classManager;
    DMenu menu;

    MenuUpdater(ClassManager cm, DMenu m) {
	classManager = cm;
	menu = m;
    }

    public void start() {
	new Thread(this,this.getClass().getName()).start();
    }

    public void run() {
	if (classManager.hasMainMethod()) {
	    menu.enable(TeikadeMenuBar.MENU_EXECUTE);
	    menu.enable(TeikadeMenuBar.MENU_EXECUTE_WITH_ARGS);
	}
	if (classManager.isApplet()) {
	    menu.enable(TeikadeMenuBar.MENU_EXECUTE);
	    menu.enable(TeikadeMenuBar.MENU_EXECUTE_WITH_ARGS);
	    menu.enable(TeikadeMenuBar.MENU_EXECUTE_WITH_HTML);
	}
    }
}

