/*
 * Copyright (c) 1995-2004 PFU Limited.
 *	author Osamu Satoh, Kazuhisa Shirakami
 */
package teikade.sys;

import teikade.util.*;
import teikade.lang.ClassSpec;
import teikade.gui.Transcript;
import teikade.gui.Browser;
import teikade.gui.dialog.TeikadeDialog;
import teikade.gui.dialog.TeikadeFileDialog;
import java.io.*;
import java.util.*;
import java.awt.Frame;

/**
 * teikade system management class
 * it manages all projects defined in this teikade system
 */
public final class SystemManager {
    /**
     * welcome message
     */
    //static final String welcome = TeikadeConstants.welcome;

    /**
     * some constant names for files/directories
     */
    static final String PROFDIRNAME = ".teikade";
    static final String BEFORE_14   = ",projects";
    static final String PROJECTS    = ".projects";

    public static boolean onSolaris = false;
    public static boolean onWinNT   = false;
    public static boolean onWin32   = false;
    public static boolean onOS2     = false;
    public static boolean onJDK1_1  = false;
    public static boolean onJDK1_2  = false;
    public static boolean needWin32TextComponentPatch = false;

    public static String javaVersion;

    public static String USERHOME      = System.getProperty("user.home").trim();
    public static String OSNAME        = System.getProperty("os.name");
    public static String JAVAVERSION   = System.getProperty("java.version");
    public static String CLASSPATH     = System.getProperty("java.class.path").trim();
    public static String PATHSEPARATOR = System.getProperty("path.separator");
    public static String TEIKADEBIN;
    static {
	if (!USERHOME.endsWith(File.separator)) {
	    USERHOME += File.separator;
	}
	initVariables();
    }

    /**
     * File object for profile directory
     */
    private static TeikadeFile profdir;

    /**
     * properties header
     */
    private final static String PROPERTIES =  ".properties";

    /**
     * system change list object
     */
    private static ChangeList changes;

    /**
     * system transcript window object: the default Frame for Dialogs
     */
    private static Transcript transcript;

    /**
     * linked list for projects managed by this SystemManager
     */
    /*private*/ static ProjectManager projects;

    private static int debugLevel;

    public static void initVariables() {
	String ver = System.getProperty("java.specification.version");
	if (ver != null) {
	    JAVAVERSION = ver;
	}
	if (OSNAME.equals("Solaris")
	 || OSNAME.equals("SunOS")
	 || OSNAME.equals("Linux")
	 || OSNAME.equals("UXP/DS")) {
	    onSolaris = true;
	} else if (OSNAME.startsWith("Windows ")) {
	    onWin32 = true;
	    if (OSNAME.equals("Windows 95")
	     || OSNAME.equals("Windows 98")
	     || OSNAME.equals("Windows Me")) {
		/* nothing to do */;
	    } else {
		onWinNT = true;
	    }
	} else if (OSNAME.equals("OS/2")) {
	    onOS2 = true;
	}
	if (Character.isDigit(JAVAVERSION.charAt(0))) {
	    if (JAVAVERSION.compareTo("1.2") >= 0) {
		onJDK1_2 = true;
		onJDK1_1 = true;
	    } else if (JAVAVERSION.compareTo("1.1") >= 0) {
		onJDK1_1 = true;
	    }
	} else if (JAVAVERSION.startsWith("jvm11_")
		|| JAVAVERSION.startsWith("sbb:")) {
	    onJDK1_1 = true;
	}
	if (onWin32
	 && JAVAVERSION.compareTo("1.1") > 0
	 && JAVAVERSION.compareTo("1.1.5") <= 0) {
	    needWin32TextComponentPatch = true;
	}
    }

    private static void updateVariables() {
	String osname = TeikadeProperties.getString("os.name");
	if (osname != null) {
	    OSNAME = osname;
	}
	String javaVersion = System.getProperty("java.version");
	if (javaVersion != null) {
	    JAVAVERSION = javaVersion;
	}
	initVariables();
	if ("true".equals(TeikadeProperties.getString("system.onSolaris"))) {
	    onSolaris = true;
	}
	if ("true".equals(TeikadeProperties.getString("system.onWin32"))) {
	    onWin32 = true;
	}
	if ("true".equals(TeikadeProperties.getString("system.onWinNT"))) {
	    onWin32 = true;
	    onWinNT = true;
	}
	if ("true".equals(TeikadeProperties.getString("system.onOS/2"))) {
	    onOS2 = true;
	}
	if ("true".equals(TeikadeProperties.getString("system.onJDK1_1"))) {
	    onJDK1_1 = true;
	}
	if ("true".equals(TeikadeProperties.getString("system.onJDK1_2"))) {
	    onJDK1_2 = true;
	}
	if ("true".equals(TeikadeProperties.getString("patch.win32.text.component"))) {
	    needWin32TextComponentPatch = true;
	}
	StringTokenizer pathes = new StringTokenizer(CLASSPATH, PATHSEPARATOR);
	String teikade = File.separator + "teikade" + File.separator;
	String TEIKADE = File.separator + "TEIKADE" + File.separator;
	while (pathes.hasMoreTokens()) {
	    String path = pathes.nextToken(), bin;
	    if (path.indexOf(teikade) >= 0 || path.indexOf(TEIKADE) >= 0) {
		bin = path + File.separator + ".." +
			     File.separator + "bin" +
			     File.separator + OSNAME.replace(' ', '_') +
			     File.separator;
		if (TeikadeFile.create(bin).exists()) {
		    TEIKADEBIN = bin;
		    break;
		}
		int space = OSNAME.indexOf(' ');
		if (space > 0) {
		    bin = path + File.separator + ".." +
				 File.separator + "bin" +
				 File.separator + OSNAME.substring(0, space) +
				 File.separator;
		    if (TeikadeFile.create(bin).exists()) {
			TEIKADEBIN = bin;
			break;
		    }
		}
	    }
	}
	TeikadeProperties.updateProperties();
    }

    /**
     * main routine for teikade
     */
    public static void main(String argv[]) {
	String dlProperty = System.getProperty("teikade.debug");
	debugLevel = dlProperty == null ? 0 : Integer.parseInt(dlProperty);

	try {
	    String profile = System.getProperty("teikade.profdir");
	    if (profile == null) {
		profile = USERHOME + PROFDIRNAME;
	    }

	    profdir = TeikadeFile.create(profile);
	    try {
		if (!profdir.exists()) {
		    if (profdir.mkdir()) {
			System.out.println("Created profile directory: " + profile);
		    } else {
			throw new java.io.IOException("Fail to create directory for teikade profile: " + profile);
		    }
		}
	    } catch (java.io.IOException e) {
		System.err.println("Could not open profile directory. Giving up.");
		System.err.println("Profile directory was: " + profile);
		e.printStackTrace();
		System.exit(1);
	    }
	    loadProperties();
	    updateVariables();
	    transcript = Transcript.getTranscript();
	    transcript.start();
	} catch (java.lang.Exception e) {
	    e.printStackTrace();
	    System.err.println("Exception is caught during start up. Exiting.");
	    System.exit(-1);
	}

	message("The system is now coming up. Wait a moment please ..."
	      + "\n\tRunning under Java " + System.getProperty("java.version")
	      + " runtime on " + System.getProperty("os.name")
	      + ".\n\n");

	PluginManager.loadPlugins();

	readProjects(true);

	messageln("\n" + TeikadeConstants.welcome);
/*
	messageln("!!!Caution!!! There are known bugs in JDK.\n"
	    + "    On Motif Window System:\n"
	    + "        Clicking mouse on CodeView doesn't work correctly.\n"
	    + "        Drag and move text cursor instead.\n"
	    + "    On Windows95/NT:\n"
	    + "        Sometimes dialogs don't layout GUI components correctly.\n"
	    + "        Resize the dialog instead.");
*/
	messageln("");

	for (int i = 0; i < argv.length; i++) {
	    ProjectManager pm = findProject(argv[i]);
	    if (pm != null) {
		Browser.openOn(pm.getCodePath());
	    } else {
		beep();
		messageln("Specified project '" + argv[i] + "' is not found in projects. Check out project name.");
	    }
	}

	// transcript.setDefaultCursor();
    }

    /**
     * beep
     */
    public static void beep() {
	System.err.print("\007");
	System.err.flush();
    }

    public static void raiseTranscript() {
	transcript.show();
    }

    /**
     * append message to the transcript window
     */
    public static void message(String msg) {
	transcript.message(msg);
    }

    /**
     * append message to the transcript window
     */
    public static void message(InputStream is) {
	transcript.message(is);
    }

    /**
     * append message followed by a newline char
     */
    public static void messageln(String msg) {
	transcript.message(msg + "\n");
    }

    /**
     * append error message to the transcript window
     */
    public static void error(String msg) {
	beep();
	transcript.error(msg);
    }

    public static void errorln(String msg) {
	error(msg + "\n");
    }

    public static void exception(String msg, Exception e) {
	System.out.println("teikade caught exception: " + msg);
	e.printStackTrace();
    }

    public static void exception(Exception e) {
	errorln(e.getMessage());
    }

    public static void debug(int level, Object obj) {
	if (debugLevel >= level) {
	    debug(obj.toString());
	}
    }

    public static void debug(Object obj) {
	messageln(obj.toString());
    }

    public static int debugLevel() {
	return debugLevel;
    }

    /**
     * information
     */
    public static void information(String message) {
	TeikadeDialog.information(getDefaultFrame(), message);
    }

    /**
     * information
     */
    public static void information(String message, String btnLabel) {
	TeikadeDialog.information(getDefaultFrame(), message, btnLabel);
    }

    /**
     * confirm Ok or Cancel
     */
    public static boolean confirm(String message) {
        return confirm(message, TeikadeDialog.OK, TeikadeDialog.CANCEL);
    }

    /**
     * confirm <label> or Cancel
     */
    public static boolean confirm(String message, String label) {
        return confirm(message, label, TeikadeDialog.CANCEL);
    }

    /**
     * confirm Ok or Cancel
     */
    public static boolean confirm(String message, String label1, String label2) {
        return TeikadeDialog.confirm(getDefaultFrame(), message, label1, label2);
    }

    /**
     * request user input for String
     */
    public static String request(String message) {
        return TeikadeDialog.request(getDefaultFrame(), message, "");
    }

    /**
     * request user input for String
     */
    public static String request(String message, String defaultString) {
        return TeikadeDialog.request(getDefaultFrame(), message, defaultString);
    }

    /**
     * request file
     */
    public static String requestFile(String message, String dir, String file) {
        return TeikadeFileDialog.request(getDefaultFrame(), message, dir, file);
    }

    public static String requestFileName(String message,
					 String directory, String file) {
        return TeikadeFileDialog.requestName(getDefaultFrame(), message,
							       directory, file);
    }

    private static Frame getDefaultFrame() {
        return transcript.getDefaultFrame();
    }

    /**
     * get the Project Manager for given name
     */
    public static ProjectManager findProject(String name) {
	if (projects != null) {
	    return (ProjectManager)projects.find(name);
	} else {
	    return null;
	}
    }

    /**
     * get the File object for profile directory
     */
    public static TeikadeFile profileDirectory() {
	return profdir;
    }

    /**
     * load properties
     */
    public static void loadProperties() throws FileNotFoundException, IOException {
	TeikadeFile properties = TeikadeFile.create(profdir, PROPERTIES);
	if (properties.exists()) {
	    InputStream is = properties.getInputStream();
	    try {
		TeikadeProperties.load(is);
	    } finally {
		is.close();
	    }
	}
    }

    /**
     * load properties
     */
    public static Properties loadProperties(String fileName)
		throws IOException {
	TeikadeFile properties = TeikadeFile.create(profdir, fileName);
	if (properties.exists()) {
	    Properties props = new Properties();
	    InputStream is = properties.getInputStream();
	    try {
		props.load(is);
	    } finally {
		is.close();
	    }
	    return props;
	} else {
	    throw new FileNotFoundException(fileName);
	}
    }

    /**
     * save property
     */
    public static void saveProperties() {
	TeikadeFile properties = TeikadeFile.create(profdir, PROPERTIES);
	try {
	    OutputStream os = properties.getOutputStream();
	    try {
		TeikadeProperties.save(os, null);
	    } finally {
		os.close();
	    }
	} catch (IOException e) {
	    messageln("Fail to save property " + properties.getAbsolutePath());
	}
    }

    /**
     * save property
     */
    public static void saveProperties(String fileName, Properties props)
	    throws IOException {
	TeikadeFile properties = TeikadeFile.create(profdir, fileName);
	OutputStream os = properties.getOutputStream();
	try {
	    props.save(os, null);
	} finally {
	    os.close();
	}
    }

   /**
     * get the File object for _projects file
     */
    public static TeikadeFile projectsFile() {
	return TeikadeFile.create(profileDirectory(), PROJECTS);
    }

   /**
     * get the File object for ,projects file
     */
    public static TeikadeFile before14projectsFile() {
	return TeikadeFile.create(profileDirectory(), BEFORE_14);
    }

   /**
     * convert ,project -> _project
     */
    public static void convertBefore14(TeikadeFile oldF, TeikadeFile newF) {
	oldF.renameTo(newF);
	// nothing to be done on contents
    }

   /**
     * get the File object for .changes file
     */
    public static TeikadeFile changesFile() {
	return TeikadeFile.create(profileDirectory(), ChangeList.CHANGES);
    }

   /**
     * get the File object for .changes file
     */
    public static TeikadeFile before14changesFile() {
	return TeikadeFile.create(profileDirectory(), ChangeList.BEFORE_14);
    }

   /**
     * get system ChangeList object
     */
    public static ChangeList changes() {
	if (changes == null) {
	    TeikadeFile chgf = changesFile();
	    TeikadeFile before14 = before14changesFile();
	    if (!chgf.exists() && before14.exists()) try {
		before14.renameTo(chgf);
	    } catch (Exception e) {
		SystemManager.information("Failed renaming ,changes -> .changes");
	    }
	    changes = new ChangeList(chgf);
	}
	return changes;
    }

    /**
     * read ,project file
     */
    static synchronized void readProjects(boolean verbose) {
	if (projects != null) {
	    disposeProjects();
	}
	TeikadeFile projsfile = projectsFile();
	if (!projsfile.exists()) {
	    TeikadeFile oldFile = before14projectsFile();
	    if (oldFile.exists()) {
		if (SystemManager.confirm("Projects file now renamed.\n" +
					  "Should I convert old one?",
					  "Please", "No, thanx")) {
		    convertBefore14(oldFile, projsfile);
		} else if (SystemManager.confirm("Exit teikade now?\n", "Yes", "No")) {
		    SystemManager.information("Okay, see you next time. bye.");
		    System.exit(0);
		}
	    }
	}
	try {
	    if (verbose) messageln("Reading projects file: " + projsfile.getPath());
	    String line;
	    String pr = null;
	    String buffer = CodeConverter.toString(projsfile);
	    DataInputStream dis = new DataInputStream(new StringBufferInputStream(buffer));
	    try {
		while ((line = dis.readLine()) != null) {
		    int spc = line.indexOf(' ');
		    if (spc < 0) {
			spc = line.length();
			line = line + " " + line;
		    }
		    pr = line.substring(0, spc);
		    String path = line.substring(spc + 1);
		    if (addProject(pr, path, false, false) != null) {
			if (verbose) messageln("\t" + pr + " found on " + path);
		    }
		}
	    } finally {
		dis.close();
	    }
	    if (verbose) {
		int numProjects = numProjects();
		if (numProjects != 0) {
		    messageln("Found " + numProjects +
			" project" + (numProjects == 1 ? "." : "s."));
		} else {
		    messageln("No project found.");
		}
	    }
	} catch(java.io.FileNotFoundException e) {
	    messageln("No projects file exists.");
	} catch(java.io.IOException e) {
	    exception("Could not read projects file.", e);
	}
    }

    /**
     * update .project file
     */
    static synchronized void updateProjects() {
	TeikadeFile projsfile = projectsFile();
	try {
	    message("Updating projects file ...");
	    PrintStream ps = new PrintStream(projsfile.getOutputStream());
	    try {
		Enumeration en = allProjects();
		while (en.hasMoreElements()) {
		    ProjectManager pm = (ProjectManager)en.nextElement();
		    ps.write(CodeConverter.toBytes(pm.name()));
		    ps.print(" ");
		    ps.write(CodeConverter.toBytes(pm.path()));
		    ps.println("");
		}
	    } finally {
		ps.close();
	    }
	    messageln(" done.");
	} catch(java.io.IOException e) {
	    exception("Could not write projects file.", e);
	}
    }

    /**
     * internal project creation routine
     * does not create project directory and profile directory
     */
    private static ProjectManager addProject(String projname, String path,
					boolean rebuild, boolean writeOnFile) {
	if (findProject(projname) != null) {
	    // never fall in this block
	    errorln("PrintStream file consistency error: Project already exists: " + projname);
	    return null;
	}
	try {
	    ProjectManager pman = new ProjectManager(projname, path);
	    if (pman != null) {
		if (projects == null) {
		    projects = pman;
		} else {
		    projects.add(pman);
		}
		Transcript.addProjectToMenu(projname);
		if (rebuild) {
		    pman.rebuild();
		} else {
		    pman.readPackagesFile();
		}
		if (writeOnFile) {
		    updateProjects();
		}
		return pman;
	    } else {
		information("Could not set up project " + projname + " on " + path);
		return null;
	    }
	} catch (java.io.IOException e) {
	    exception("SystemManager#addProject(String,String,boolean,boolean)", e);
	    return null;
	}
    }

    /**
     * add project entry
     */
    public static ProjectManager createProject(String projname, String path, boolean rebuild) {
	if (findProject(projname) != null) {
	    information("Project already exists: " + projname);
	    return null;
	}
	TeikadeFile projectDir = TeikadeFile.create(path);
	if (!projectDir.exists()) {
	    boolean create = confirm("Directory " + path + " not found. Create it?", "Create");
	    if (!create) return null;
	    projectDir.mkdirs();
	}
	TeikadeFile profDir = TeikadeFile.create(projectDir, PROFDIRNAME);
	if (!profDir.exists()) {
	    boolean create = confirm("Information about project should go under this directory?\n" +
				     "Or under your home directory?",
				     "Project directory", "Home directory" );
	    if (create) profDir.mkdir();
	}
	return addProject(projname, path, rebuild, true);
    }

    /**
     * set projects entry
     */
    public static void setProjects(ProjectManager pman) {
	projects = pman;
    }

    /**
     * get projects entry
     */
    public static ProjectManager getProjects() {
	return projects;
    }

    /**
     * enumerate all ProjectManagers in the system
     */
    public static Enumeration allProjects() {
	return new ManagerEnumerator(projects);
    }

    /**
     * enumerate all ProjectManager's names in the system
     */
    public static Enumeration allProjectNames() {
	return new ManagerEnumerator(projects, true);
    }

    /**
     * returns number of projects in the System
     */
    public static int numProjects() {
	if (projects != null) {
	    return projects.numBrothers() + 1;
	} else {
	    return 0;
	}
    }

    /**
     * dispose all ProjectManagers
     */
    private static void disposeProjects() {
	projects.dispose();
	projects = null;
    }

    /**
     * sync all projects
     */
    public static void sync() {
	for (Enumeration p = allProjects(); p.hasMoreElements(); ) {
	    ((ProjectManager)p.nextElement()).sync();
	}
    }
}
