/*
 * Copyright (c) 1998 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.TeikadeProperties;
import teikade.sys.CodePath;

public class CompileFactory {
    // project name for user default compile
    public static final String USER_DEFAULT = "default";

    private static CompileFactory instance;

    public static CompileFactory getInstance() {
	if (instance == null) {
	    try {
		instance = (CompileFactory)newInstance("compile.factory");
	    } catch (Exception e) {
	    }
	    if (instance == null) {
		instance = new CompileFactory();
	    }
	}
	return instance;
    }

    protected static Object newInstance(String propertyName) {
	String name = TeikadeProperties.getParsedString(propertyName);
	if (name != null) {
	    try {
		return Class.forName(name).newInstance();
	    } catch (Exception e) {
	    }
	}
	return null;
    }

    public static String getCompilePropertyName(String projectName) {
	if (projectName == null) {
	    return null;
	} else if (USER_DEFAULT.equals(projectName)) {
	    return "comile.builder";
	} else {
	    return "comile.builder." + projectName;
	}
    }

    protected CompileFactory() {
    }

    public Compile create(String projectName) {
	Compile compile;
	if ((projectName != null && !USER_DEFAULT.equals(projectName)
	    && (compile = createForProject(projectName)) != null)
	 || (compile = createForUser()) != null
	 || (compile = createForSystem()) != null) {
	    return compile;
	} else {
	    return null;
	}
    }

    protected Compile createForProject(String projectName) {
	try {
	    return (Compile)newInstance(getCompilePropertyName(projectName));
	} catch (Exception e) {
	    return null;
	}
    }

    protected Compile createForUser() {
	try {
	    // compile.builder is  for compatible old version of teikade
	    return (Compile)newInstance(getCompilePropertyName(USER_DEFAULT));
	} catch (Exception e) {
	    return null;
	}
    }

    protected Compile createForSystem() {
	if (CompileJDKNoWindow.canExecute()) {
	    return new CompileJDKNoWindow();
	} else if (CompileJDK.canExecute()) {
	    return new CompileJDK();
	} else {
	    return null;
	}
    }
}
