/*
 * Copyright (c) 1999 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import java.awt.*;
import java.util.Hashtable;
import java.util.Enumeration;
import teikade.gui.component.*;
import teikade.sys.SystemManager;
import teikade.sys.ProjectManager;

public class CompilePropertiesEditor extends Panel {
    private static final CompilePropertiesEditor
	instance = new CompilePropertiesEditor();

    private DChoice projectSelector;
    private Panel projectPanel;
    private CardLayout projectLayout;

    public static CompilePropertiesEditor on(Container parent) {
	CompilePropertiesEditor editor = new CompilePropertiesEditor();
	editor.init();
	parent.setLayout(new BorderLayout());
	parent.add("North", editor);
	return editor;
    }

    public void init() {
	/* create project name selector and panel */
	projectSelector = new DChoice();
	projectPanel = new Panel();
	projectLayout = new CardLayout();
	projectPanel.setLayout(projectLayout);
	addCard("system default", null);
	addCard("user default", CompileFactory.USER_DEFAULT);
	Enumeration projects = SystemManager.allProjects();
	while (projects.hasMoreElements()) {
	    ProjectManager project = (ProjectManager)projects.nextElement();
	    String name = project.name();
	    addCard("project " + name, name);
	}

	/* layout project name selector */ 
	Panel selectorPanel = new Panel();
	selectorPanel.setLayout(new BorderLayout());
	selectorPanel.add("West", projectSelector);

	/* layout components */
	setLayout(new BorderLayout());
	if (projectSelector.countItems() > 0) {
	    add("North",  selectorPanel);
	    add("Center", projectPanel);
	} else {
	    add("North", new DLabel("No compiler for Java in this envilonment."));
	}
    }

    protected void addCard(String label, String projectName) {
	Compile compile = Compile.getInstance(projectName);
	if (compile != null) {
	    CompilePropertiesEditorCard card =
		new CompilePropertiesEditorCard(compile.getCompileProperties());
	    card.init(projectName);
	    projectSelector.addItem(label);
	    projectPanel.add(label, card);
	}
    }

    public String check() {
	int num = projectPanel.countComponents();
	for (int i = 0; i < num; i++) {
	    Component comp = projectPanel.getComponent(i);
	    String error = ((CompilePropertiesEditorCard)comp).check();
	    if (error != null) {
		return error;
	    }
	}
	return null;
    }

    public void accept() {
	int num = projectPanel.countComponents();
	for (int i = 0; i < num; i++) {
	    Component comp = projectPanel.getComponent(i);
	    ((CompilePropertiesEditorCard)comp).accept();
	}
    }

    public boolean action(Event event, Object what) {
	if (event.target == projectSelector) {
	    projectLayout.show(projectPanel, projectSelector.getSelectedItem());
	    return true;
	}
	return false;
    }
}

class CompilePropertiesEditorCard extends Panel {
    CompileProperties properties;
    private Hashtable fields = new Hashtable();

    public CompilePropertiesEditorCard(CompileProperties properties) {
	this.properties = properties;
    }

    public void init(String projectName) {
	Panel labels = new Panel();
	Panel values = new Panel();
	labels.setLayout(new GridLayout(0, 1));
	values.setLayout(new GridLayout(0, 1));

	Enumeration names = properties.getNames();
	while (names.hasMoreElements()) {
	    String name = (String)names.nextElement();
	    labels.add(new DLabel(properties.getLabel(name), DLabel.RIGHT));
	    values.add(createField(name, properties));
	}

	Panel defaultPanel = new Panel();
	defaultPanel.setLayout(new BorderLayout());
	defaultPanel.add("West", labels);
	defaultPanel.add("Center", values);

	setLayout(new BorderLayout());
	add("North", defaultPanel);
    }

    protected TextField createField(String name, CompileProperties properties) {
	DTextField field = new DTextField(20);
	String value = properties.getString(name);
	if (value == null) {
	    value = properties.getDefaultString(name);
	}
	field.setText(value);
	field.setEditable(properties.isEditable());
	fields.put(name, field);
	return field;
    }

    public String check() {
	return null;
    }

    public void accept() {
	Enumeration keys = fields.keys();
	while (keys.hasMoreElements()) {
	    String name = (String)keys.nextElement();
	    TextField field = (TextField)fields.get(name);
	    String value = field.getText();
	    String current = properties.getString(name);
	    if (current == null && !value.equals(properties.getDefaultString(name))
	     || current != null && !value.equals(current)) {
		properties.put(name, value);
	    }
	}
    }
}
