/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.util;

import teikade.sys.*;
import java.io.*;

public class SourceCodeRange {
    protected int startPoint;
    protected int endPoint;
    protected int startLine;
    protected int endLine;

    private static final Object cacheLock = new Object();
    private static long cacheModified;
    private static TeikadeFile cacheFile;
    private static String cacheCode;

    public SourceCodeRange(int startPoint, int endPoint, int startLine, int endLine) {
	this.startPoint = startPoint;
	this.endPoint = endPoint;
	this.startLine = startLine;
	this.endLine = endLine;
    }

    public SourceCodeRange(int startPoint, int endPoint) {
	this.startPoint = startPoint;
	this.endPoint = endPoint;

	// line numbers are unknown
	this.startLine = -1;
	this.endLine = -1;
    }

     SourceCodeRange(SourceCodeRange clone) {
	startPoint = clone.startPoint;
	endPoint   = clone.endPoint;
	startLine  = clone.startLine;
	endLine    = clone.endLine;
    }

    public int startPoint() {
	return startPoint;
    }

    public int endPoint() {
	return endPoint;
    }

    public int startLine() {
	return startLine;
    }

    public int endLine() {
	return endLine;
    }

    public int length() {
	return endPoint - startPoint;
    }

    public int lines() {
	return endLine - startLine;
    }

    public String getCode(TeikadeFile sourceFile) {
	synchronized (cacheLock) {
	    long lastModified = sourceFile.lastModified();
	    if (cacheModified < lastModified || !sourceFile.equals(cacheFile)) {
		cacheFile = sourceFile;
		cacheModified = lastModified;
		try {
		    cacheCode = CodeConverter.toString(sourceFile);
		} catch (java.io.IOException e) {
		    cacheFile = null;
		    SystemManager.exception("Could not get code. Sorry.", e);
		    return "/* COULD NOT GET CODE */";
		}
	    }
	    return cacheCode.substring(startPoint, endPoint);
	}
    }

    public boolean isSynced() {
	return true;
    }

    public boolean isChanged() {
	return false;
    }

    public void synced() {}

    public void dispose() {}
}
